/*
 * ar3cmonitor - ledwidget.cpp
 *
 * Copyright (C) 2004  Szymon Acedanski <accek@podlesie.net>
 * Copyright (C) 2006..2011  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <qpainter.h>

#include "ledwidget.h"
#include "control.h"
#include <cdata.h>

LedWidget::LedWidget(int _physical, QColor _activecolor, QColor _inactivecolor,
		QWidget *parent, const char *name, Qt::WFlags f)
	: BufferedWidget(parent, name, f)
{
	setMinimumHeight(19);
	setMinimumWidth(30);

	activecolor = _activecolor;
	inactivecolor = _inactivecolor;
	physical = _physical;
	unknown = false;
}

void LedWidget::paint(QPainter *p)
{
	int x = width() / 2;
	int y = height() / 2;
	int undefined = 1;
	int value = 0;

	/*
	 * FIXME: remove unknown?
	 */
	if (!unknown && (physical != 0xffff)) {
		if (!cdata_get_binary(physical, &value))
			undefined = 0;
	}

	if (undefined) {
		p->setPen(Qt::black);
		p->setBrush(inactivecolor);
	} else {
		p->setPen(QColor(0, 0, 150, QColor::Hsv));
		p->setBrush(value ? activecolor : inactivecolor);
	}

	p->drawEllipse(x - 8, y - 8, 17, 17);

	if (CO->reviewMode() || CO->offlineMode()) {
		p->drawLine(x - 3, y - 3, x + 3, y + 3);
		p->drawLine(x - 3, y + 3, x + 3, y - 3);
	}

	if (undefined) {
		p->setPen(Qt::white);
		p->drawText(1, 0, width(), height(), Qt::AlignCenter,
				QString("?"));
	}
}

void LedWidget::haveDataChanged()
{
	unknown = !CO->haveData();
	update();
}

QSize LedLabel::minimumSizeHint(void) const
{
	int maxw = fontMetrics().width("MMMMM...");
	int w = fontMetrics().width(text);

	if (w > maxw)
		w = maxw;
	return QSize(w, fontMetrics().height() + 2);
}

QSize LedLabel::sizeHint(void) const
{
	return QSize(fontMetrics().width(text), fontMetrics().height() + 2);
}

QSizePolicy LedLabel::sizePolicy(void) const
{
	return QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
}

LedLabel::LedLabel(int _physical, const QString &caption, QColor _activecolor,
		QWidget *parent, const char *name, Qt::WFlags f)
	: BufferedWidget(parent, name, f)
{
	activecolor = _activecolor;
	physical = _physical;
	text = caption;
	width_cached = -1;

	updateGeometry();
}

void LedLabel::generate_text(void)
{
	unsigned int len = text.length();

	if (width_cached == width())
		return;

	short_text = text;

	while (fontMetrics().width(short_text) > width()) {
		len--;
		short_text = text.left(len).append("...");
	}
}

void LedLabel::paint(QPainter *p)
{
	int undefined = 1;
	int value = 0;

	if (physical != 0xffff) {
		if (!cdata_get_binary(physical, &value))
			undefined = 0;
	}

	if (!undefined && value) {
		p->setPen(activecolor);
	} else {
		p->setPen(QColor(0, 0, 185, QColor::Hsv));
	}

	generate_text();
	p->drawText(0, 1, width(), height() - 1,
			Qt::AlignLeft | Qt::AlignVCenter, short_text);
}
