//
// C++ Interface: main
//
// Description:
//
//
// Author: Szymon Acedanski <accek@podlesie.net>, (C) 2004
// Copyright (C) 2005..2016  Krzysztof Mazur <krzysiek@podlesie.net>
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "ar3cmonitor.h"
#include "commobj.h"
#include <stdio.h>
#include <profile.h>
#include <fonts.h>
#include <settime.h>

#ifdef TOUCH_SCREEN
#define TOUCHSCREEN_DEFAULT	1
#else
#define TOUCHSCREEN_DEFAULT	0
#endif

int profile = 0;
int show_points_count = 0;
unsigned long mem_low = 0;
unsigned long mem_high = 0;
unsigned int mreq_limit = MREQ_LIMIT_DEFAULT;
unsigned int ar3cmon_update_interval = AR3CMON_UPDATE_INTERVAL;
unsigned int ar3cmon_offline_update_interval = AR3CMON_OFFLINE_UPDATE_INTERVAL;
const char *config_encoding = "ISO-8859-2";
char *lang = NULL;
char *conf_locale = NULL;
int led_no_symbol = 0;
unsigned int num_layers = 0;
int font_magnify;
int touchscreen = TOUCHSCREEN_DEFAULT;

#include <qapplication.h>
#if QT_VERSION >= 0x40000
#include <QTranslator>
#include <QLocale>
#elif QT_VERSION > 0x30000
#include <qlocale.h>
#endif
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include "version.h"

extern const char *ar3cmon_host;
extern unsigned short ar3cmon_port;
const char *pin;
static int mem_autotune = 1;
int acache_opt = 1;
int scheme;
int show_menu = 1;
int show_changesvr = 1;
int show_statusbar = 1;
char *password;
int swap_cursors_style;
#ifdef __AR4C__
const char *ts_calibrate = "sudo /ar3c/bin/ts-calib&";
#else
const char *ts_calibrate;
#endif

const char *key_q_action;
const char *key_x_action;

QWidget *SS;

#ifndef __WIN32__
extern "C" {
	void unix_init(void);
};
#else
#define unix_init() do { } while (0)
#endif

void show_version(void)
{
	printf("ar3cmonitor " AR3CMON_VERSION_STRING "\n"
"Copyright (C) 2004..2006  Szymon Acedanski\n"
"Copyright (C) 2008..2011  Krzysztof Kasperski <casper@podlesie.net>\n\n"
"Copyright (C) 2005..2016  Krzysztof Mazur <krzysiek@podlesie.net>\n\n"
"This program comes with ABSOLUTELY NO WARRANTY; see COPYING for details.\n"
"This is free software, and you are welcome to redistribute it\n"
"under certain conditions; see COPYING for details.\n");
	exit(EXIT_SUCCESS);
}

void show_help(char *arg)
{
	printf("ar3cmonitor " AR3CMON_VERSION_STRING "\n"
"Usage: %s [options] [host] [port]\n"
"Options:\n"
"      --[no]-acacheopt	enable AnalogChache optimizations\n"
"      --conf-locale	set configuration language\n"
"  -c, --show-count	show number of points in analoggraph\n"
"      --encoding	set config encoding\n"
"      --font-magnify=M	set font magnification (multiplied by 1000)\n"
"      --fullscreen	fullscreen mode\n"
"  -h, --help		show this help\n"
"      --key-q-action=F	execute specified file on 'q' keypress\n"
"      --key-x-action=F	execute specified file on 'x' keypress\n"
"      --lang		set user interface language\n"
"      --layers		enable support for layers (EXPERIMENTAL)\n"
"      --led-no-symbol	don't show symbol in ledwidget\n"
"      --maximize	show maximized\n"
"      --mem low high	set low and high memory mark\n"
"  -m, --menu		show menubar\n"
"      --no-menu\t	hide menubar\n"
"      --no-changesvr	disable possibility of changing server\n"
"      --no-statusbar	disable statusbar\n"
"      --no-touchscreen	disable touchscreen friendly mode\n"
"  -n, --scheme		use scheme (0 by default, -1 for without scheme init)\n"
"      --off-update-ival\n"
"			change offline mode update interval (ms)\n"
"      --password	use specified password for anonymous user\n"
"      --pin-action=P:F	execute specified file F on P pin\n"
"      --profile\t	enable profiling code\n"
"      --rlimit limit	set request limit (default %d)\n"
"  -s, --swap-cursors	swap cursor style (0 - none, 1 - menu, 2 - images, default none)\n"
"      --touchscreen	enable touchscreen friendly mode\n"
"      --ts-calibrate=C	enable touchscreen calibration using specified command\n"
"      --update-ival	change update interval (ms)\n"
"  -V, --version\t	show version\n",
			(arg != NULL) ? arg : "ar3cmonitor",
			MREQ_LIMIT_DEFAULT);
	exit(EXIT_SUCCESS);
}

static void show_mem_settings(void)
{
	fprintf(stderr, "memory settings: low = %ldkB, high = %ldkB\n",
			mem_low / 1024, mem_high / 1024);
}

static void ExpectedMemoryUsageTune(void)
{
	if (mem_low > 0)
		return;

#ifdef MEM_AUTOTUNE
	if (mem_autotune) {
		unsigned long long ps;
		long pages, page_size;

		page_size = sysconf(_SC_PAGESIZE);
		if (page_size == -1)
			goto err;
		pages = sysconf(_SC_PHYS_PAGES);
		if (pages == -1)
			goto err;
		ps = (unsigned long long) pages * page_size;

		mem_low = AR3CMON_MEM_LOW_AUTOTUNE(ps);
		mem_high = AR3CMON_MEM_HIGH_AUTOTUNE(ps);
		show_mem_settings();
		return;
	}
err:
#endif
	mem_low = AR3CMON_MEM_LOW;
	mem_high = AR3CMON_MEM_HIGH;
}

static void fix_mem_setting(void)
{
	int print_res = 0;

	if (mem_low < AR3CMON_MEM_MIN) {
		mem_low = AR3CMON_MEM_MIN;
		fprintf(stderr, "Too small mem_low. Setting %ld.\n", mem_low);
		print_res = 1;
	}
	if (mem_high < mem_low) {
		mem_high = mem_low;
		print_res = 1;
	}
	if (print_res)
		show_mem_settings();
}

int screensaver_enabled = 0;

#ifndef __WIN32__

extern "C" {
	void show_screensaver(int);
	void hide_screensaver(int);
};

void show_screensaver(int)
{
	MainWindow->hide();
	SS->showFullScreen();
	SS->show();
	screensaver_enabled = 1;
}

void hide_screensaver(int)
{
	SS->hide();
	MainWindow->show();
	screensaver_enabled = 0;
}
#endif

static void parse_pin_action(const char *arg)
{
	char *t;
	char *s;

	t = strdup(arg);
	if (!t)
		abort();
	s = strchr(t, ':');
	if (!s) {
		fprintf(stderr, "pin-action: missing ':' in argument\n");
		exit(EXIT_FAILURE);
	}
	s[0] = '\0';
	add_pin_action(t, s + 1);
}

QApplication *qa;

int main(int argc, char **argv)
{
	int fullscreen = 0;
	int maximize = 0;
	int host_specified = 0;
	int port_specified = 0;
	int arg = 1;

	while (arg < argc) {
		if (!strcmp(argv[arg], "--fullscreen"))
			fullscreen = 1;
		else if (!strcmp(argv[arg], "--maximize"))
			maximize = 1;
		else if (!strcmp(argv[arg], "--mem")) {
			if (arg + 2 >= argc) {
				fprintf(stderr, "missing argument for --mem"
						" option.\n");
				exit(EXIT_FAILURE);
			}
			mem_low = atol(argv[arg + 1]);
			mem_high = atol(argv[arg + 2]);
			arg += 2;
			fix_mem_setting();
		} else if (!strcmp(argv[arg], "--rlimit")) {
			arg++;
			if (arg >= argc) {
				fprintf(stderr, "missing argument for --rlimit"
						" option.\n");
				exit(EXIT_FAILURE);
			}
			mreq_limit = atoi(argv[arg]);
		} else if (!strcmp(argv[arg], "--font-magnify")) {
			arg++;
			if (arg >= argc) {
				fprintf(stderr, "missing argument for "
						"--font-magnify option.\n");
				exit(EXIT_FAILURE);
			}
			font_magnify = atoi(argv[arg]);
			if (font_magnify < 0) {
				fprintf(stderr, "font magnification must be \n"
						"non-negative\n");
				exit(EXIT_FAILURE);
			}
		} else if (!strcmp(argv[arg], "--scheme")) {
			arg++;
			if (arg >= argc) {
				fprintf(stderr, "missing argument for --scheme"
						" option.\n");
				exit(EXIT_FAILURE);
			}
			scheme = atoi(argv[arg]);
		} else if (!strcmp(argv[arg], "--menu")
				|| !strcmp(argv[arg], "-m")) {
			show_menu = 1;
		} else if (!strcmp(argv[arg], "--no-menu")) {
			show_menu = 0;
		} else if (!strcmp(argv[arg], "--touchscreen")) {
			touchscreen = 1;
		} else if (!strcmp(argv[arg], "--no-touchscreen")) {
			touchscreen = 0;
		} else if (!strcmp(argv[arg], "--no-statusbar")) {
			show_statusbar = 0;
		} else if (!strcmp(argv[arg], "--no-changesvr")) {
			show_changesvr = 0;
		} else if (!strcmp(argv[arg], "--acacheopt")) {
			acache_opt = 1;
		} else if (!strcmp(argv[arg], "--no-acacheopt")) {
			acache_opt = 0;
		} else if (!strcmp(argv[arg], "--led-no-symbol")) {
			led_no_symbol = 1;
		} else if (!strcmp(argv[arg], "--encoding")) {
			if (arg + 1 >= argc) {
				fprintf(stderr, "missing argument for "
						"--encoding option.\n");
				exit(EXIT_FAILURE);
			}
			config_encoding = argv[++arg];
		} else if (!strcmp(argv[arg], "--password")) {
			if (arg + 1 >= argc) {
				fprintf(stderr, "missing argument for "
						"--password option.\n");
				exit(EXIT_FAILURE);
			}
			password = argv[++arg];
		} else if (!strcmp(argv[arg], "--pin")) {
			if (arg + 1 >= argc) {
				fprintf(stderr, "missing argument for "
						"--pin option.\n");
				exit(EXIT_FAILURE);
			}
			pin = argv[++arg];
			int tmp = atoi(pin);
			if (tmp < 1000 || tmp > 999999999) {
				fprintf(stderr, "invalid pin. Pin must be "
						"decimal value 4 to 9 digits "
						"long.\n");
				exit(EXIT_FAILURE);
			}
		} else if (!strcmp(argv[arg], "--lang")) {
			if (arg + 1 >= argc) {
				fprintf(stderr, "missing argument for "
						"--lang option.\n");
				exit(EXIT_FAILURE);
			}
			lang = argv[++arg];
		} else if (!strcmp(argv[arg], "--conf-locale")) {
			if (arg + 1 >= argc) {
				fprintf(stderr, "missing argument for "
						"--conf-locale option.\n");
				exit(EXIT_FAILURE);
			}
			conf_locale = argv[++arg];
		} else if (!strcmp(argv[arg], "--update-ival")) {
			if (arg + 1 >= argc) {
				fprintf(stderr, "missing argument for "
						"--update-ival option.\n");
				exit(EXIT_FAILURE);
			}
			ar3cmon_update_interval = atol(argv[arg + 1]);
			arg++;
		} else if (!strcmp(argv[arg], "--off-update-ival")) {
			if (arg + 1 >= argc) {
				fprintf(stderr, "missing argument for "
						"--off-update-ival option.\n");
				exit(EXIT_FAILURE);
			}
			ar3cmon_offline_update_interval = atol(argv[arg + 1]);
			arg++;
		} else if (!strcmp(argv[arg], "--swap-cursors")
				|| !strcmp(argv[arg], "-s")) {
			arg++;
			if (arg >= argc) {
				fprintf(stderr, "missing argument for "
						"--swap-cursors option.\n");
				exit(EXIT_FAILURE);
			}
			swap_cursors_style = atoi(argv[arg]);
		} else if (!strcmp(argv[arg], "--ts-calibrate")) {
			arg++;
			if (arg >= argc) {
				fprintf(stderr, "missing argument for "
						"--ts-calibrate option.\n");
				exit(EXIT_FAILURE);
			}
			ts_calibrate = strdup(argv[arg]);
		} else if (!strcmp(argv[arg], "--show-count")
				|| !strcmp(argv[arg], "-c")) {
			show_points_count = 1;
		} else if (!strcmp(argv[arg], "--help")
				|| !strcmp(argv[arg], "-h")) {
			show_help(argv[0]);
		} else if (!strcmp(argv[arg], "--version")
				|| !strcmp(argv[arg], "-V")) {
			show_version();
		} else if (!strcmp(argv[arg], "--profile")) {
			profile = 1;
		} else if (!strncmp(argv[arg], "--layers=", 9)) {
			int x = atoi(argv[arg] + 9);

			if (x < 0)
				x = 8;
			num_layers = x;
		} else if (!strcmp(argv[arg], "--layers")) {
			num_layers = 8;
		} else if (!strcmp(argv[arg], "--key-x-action")) {
			arg++;
			if (arg >= argc) {
				fprintf(stderr, "missing argument for "
						"--key-x-action option.\n");
				exit(EXIT_FAILURE);
			}
			if (key_x_action)
				free((void *) key_x_action);
			key_x_action = strdup(argv[arg]);
		} else if (!strcmp(argv[arg], "--key-q-action")) {
			arg++;
			if (arg >= argc) {
				fprintf(stderr, "missing argument for "
						"--key-q-action option.\n");
				exit(EXIT_FAILURE);
			}
			if (key_q_action)
				free((void *) key_q_action);
			key_q_action = strdup(argv[arg]);
		} else if (!strcmp(argv[arg], "--pin-action")) {
			arg++;
			if (arg >= argc) {
				fprintf(stderr, "missing argument for "
						"--pin-action option.\n");
				exit(EXIT_FAILURE);
			}
			parse_pin_action(argv[arg]);
		} else if (!host_specified) {
			ar3cmon_host = strdup(argv[arg]);
			host_specified = 1;
		} else if (!port_specified) {
			ar3cmon_port = atoi(argv[arg]);
			port_specified = 1;
		}
		arg++;
	}

	unix_init();

	ExpectedMemoryUsageTune();

	if (!ar3cmon_host)
		ar3cmon_host = strdup(AR3CMON_DFLT_HOST);
	printf("host=%s port=%d\n", ar3cmon_host, ar3cmon_port);

	qa = new QApplication(argc, argv);


#if QT_VERSION > 0x30000
	QString locale = lang ? QString(lang) : QLocale::system().name();
#else
	QString locale = lang ? lang : "pl";
#endif

	bool loaded;

	QTranslator translator(NULL);
	loaded = translator.load("ar3cmonitor_" + locale, AR3CMONITOR_DIR);
	if (!loaded)
		loaded = translator.load("qt_" + locale);
	if (!loaded)
		loaded = translator.load("ar3cmonitor_" + locale);
	qa->installTranslator(&translator);

	if (!font_magnify)
		font_magnify = 1000;
	fonts_init();

	MainWindow = new Ar3cMonitor(NULL);
	qa->setMainWidget(MainWindow);
	SS = new QWidget();

	if (fullscreen)
		MainWindow->showFullScreen();
	else if (maximize)
		MainWindow->showMaximized();
	else
		MainWindow->show();
	if (password != NULL)
		TheCommObject->set_password(password);
	return qa->exec();
}
