/*
 * ar3cmonitor - niceformat.c 
 * Copyright (C) 2009  Krzysztof Mazur <krzysiek@podlesie.net>
 * Copyright (C) 2009  Krzysztof Kasperski <casper@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Fundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <niceformat.h>
#include <stdio.h>
#include <math.h>

#define min(a,b) ((a) < (b) ? (a) : (b))

static int update_min_width(int width, float v)
{
	int t;
	float av;

	av = fabs(v);

	t = (int) ((av < 1) ? 0 : ceilf(log10f(av)));
	if (v < 0)
		t++;

	if (t > width)
		width = t;

	return width;
}

static int calc_prec(prec_t prec, float unit_value, int width)
{
	int precision;
	unsigned int digits;
	digits = prec & PREC_DIGITS_MASK;

	if (prec & PREC_SDIGITS) {
		if (digits) {
			precision = digits - width;
			if (precision < 0)
				precision = 0;
		} else {
			precision = (int) ceilf(log10f(fabs(unit_value)));
			if (precision < 0)
				precision = 0;
		}
	} else {
		precision = digits;
	}

	return precision;
}

void niceformat(char *buf, size_t size, float unit_value, float min_v,
		float max_v, float v, prec_t prec)
{
	int width = 1;
	int precision;

	width = update_min_width(width, min_v);
	width = update_min_width(width, max_v);

	precision = calc_prec(prec, unit_value, width);
	if (precision)
		width += precision + 1;

	snprintf(buf, size, "%*.*f", width, precision, v);
	buf[size - 1] = '\0';
}

void str_niceformat(char *buf, size_t size, float unit_value, float min_v, 
		float max_v, prec_t prec)
{
	unsigned int width = 1;
	int precision;
	size_t obuf = 0;
	unsigned int i;
	
	if (size <= 0)
		return;
	if (size == 1) {
		buf[0] = '\0';
		return;
	}

	width = update_min_width(width, min_v);
	width = update_min_width(width, max_v);

	precision = calc_prec(prec, unit_value, width);

	if (min_v < 0)
		buf[obuf++] = '-';
	
	for (i = 0; i < width && obuf < size - 1; i++) 
		buf[obuf++] = '9';

	if (precision) {
		if (obuf < size - 1)
			buf[obuf++] = '.';
		
		for (i = 0; i < (unsigned int) precision && obuf < size - 1; i++) 
			buf[obuf++] = '9';
	}

	buf[obuf++] = '\0';
}
