/*
 * ar3cmonitor - ledwidget.cpp
 *
 * Copyright (C) 2004  Szymon Acedanski <accek@podlesie.net>
 * Copyright (C) 2006..2011  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <qpainter.h>

#include "onoffwidget.h"
#include "control.h"
#include <cdata.h>

OnOffLabel::OnOffLabel(const QString &caption, QColor _activecolor,
		QColor _inactivecolor, QWidget *parent, const char *name,
		Qt::WFlags f)
	: BufferedWidget(parent, name, f)
{
	setMinimumHeight(fontMetrics().height() + 2);
	setMinimumWidth(fontMetrics().width(caption));
	activecolor = _activecolor;
	inactivecolor = _inactivecolor;
	Text = caption;
	active = false;
}

void OnOffLabel::paint(QPainter *p)
{
	p->setPen(active ? activecolor : inactivecolor);
	p->drawText(0, 1, width(), height() - 1,
			Qt::AlignHCenter | Qt::AlignVCenter, Text);
}

void OnOffLabel::setText(const QString &caption)
{
	if (caption == Text)
		return;

	Text = caption;
	update();
}

void OnOffLabel::setActiveColor(QColor c)
{
	if (c == activecolor)
		return;
	activecolor = c;
	update();
}

void OnOffLabel::setInActiveColor(QColor c)
{
	if (c == inactivecolor)
		return;
	inactivecolor = c;
	update();
}

void OnOffLabel::on()
{
	if (active == true)
		return;
	active = true;
	update();
}

void OnOffLabel::off()
{
	if (active == false)
		return;
	active = false;
	update();
}

void OnOffLabel::mousePressEvent(QMouseEvent *e)
{
	emit clicked();
}

QString OnOffLabel::text()
{
	return Text;
}
