/*
 * ar3cmonitor - settime.cpp
 *
 * 	Set datalogger time
 * 
 * Copyright (C) 2006, 2007, 2008  Krzysztof Mazur <krzysiek@podlesie.net>
 * Copyright (C) 2008  Krzysztof Kasperski <casper@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <unistd.h>

#include "ar3cmonitor.h"
#include "consts.h"
#include "commobj.h"

#include <qlabel.h>
#include <qpainter.h>
#include <qmessagebox.h>
#include <qfont.h>

#include "settime.h"

extern const char *pin;

struct pin_action {
	const char *pin;
	const char *command;
	struct pin_action *next;
};
static struct pin_action *pin_action;

int add_pin_action(const char *pin, const char *command)
{
	struct pin_action *a;

	a = (struct pin_action *) malloc(sizeof(*a));
	if (!a)
		return -ENOMEM;

	a->pin = strdup(pin);
	a->command = strdup(command);
	a->next = pin_action;
	pin_action = a;
	return 0;
}

SetTime::SetTime(unsigned long long time, QWidget *parent)
	: QDialog(parent)
{
	QDateTime t;

	QFont font(CO->hugeFont());
	
	t.setTime_t(time / 1000000000ULL);

	vl = new QVBoxLayout(this);
	
	lb = new QLabel(trUtf8("<center><b>Enter Pin:</b></center>"), this);
	le = new QLineEdit(this);
	le->setEchoMode(QLineEdit::Password);
	le->hide();
	connect(le, SIGNAL(textChanged(const QString&)), this, SLOT(check_pin(const QString&)));

	h0 = new QHBoxLayout(vl);

	h0->addWidget(lb);
	h0->addWidget(le);

	h1 = new QHBoxLayout(vl);

	key1 = new QPushButton(tr("1"), this);
	key1->setFont(font);
	key1->setMinimumHeight(50);
	connect(key1, SIGNAL(clicked()), this, SLOT(key1_clicked()));
	h1->addWidget(key1);
	key2 = new QPushButton(tr("2"), this);
	key2->setFont(font);
	key2->setMinimumHeight(50);
	connect(key2, SIGNAL(clicked()), this, SLOT(key2_clicked()));
	h1->addWidget(key2);
	key3 = new QPushButton(tr("3"), this);
	key3->setFont(font);
	key3->setMinimumHeight(50);
	connect(key3, SIGNAL(clicked()), this, SLOT(key3_clicked()));
	h1->addWidget(key3);

	h2 = new QHBoxLayout(vl);

	key4 = new QPushButton(tr("4"), this);;
	key4->setFont(font);
	key4->setMinimumHeight(50);
	connect(key4, SIGNAL(clicked()), this, SLOT(key4_clicked()));
	h2->addWidget(key4);
	key5 = new QPushButton(tr("5"), this);;
	key5->setFont(font);
	key5->setMinimumHeight(50);
	connect(key5, SIGNAL(clicked()), this, SLOT(key5_clicked()));
	h2->addWidget(key5);
	key6 = new QPushButton(tr("6"), this);;
	key6->setFont(font);
	key6->setMinimumHeight(50);
	connect(key6, SIGNAL(clicked()), this, SLOT(key6_clicked()));
	h2->addWidget(key6);

	h3 = new QHBoxLayout(vl);

	key7 = new QPushButton(tr("7"), this);;
	key7->setFont(font);
	key7->setMinimumHeight(50);
	connect(key7, SIGNAL(clicked()), this, SLOT(key7_clicked()));
	h3->addWidget(key7);
	key8 = new QPushButton(tr("8"), this);;
	key8->setFont(font);
	key8->setMinimumHeight(50);
	connect(key8, SIGNAL(clicked()), this, SLOT(key8_clicked()));
	h3->addWidget(key8);
	key9 = new QPushButton(tr("9"), this);;
	key9->setFont(font);
	key9->setMinimumHeight(50);
	
	connect(key9, SIGNAL(clicked()), this, SLOT(key9_clicked()));
	h3->addWidget(key9);

	dte = new QDateTimeEdit(t, this);
	dte->setFont(font);
#if QT_VERSION >= 0x40000
	dte->setDisplayFormat("dd-MM-yyyy hh:mm:ss");
#else
	dte->dateEdit()->setOrder(QDateEdit::YMD);
	dte->dateEdit()->setSeparator("-");
#endif
	/* Qt 3.3.3b incorect size bug workaround */
	dte->setMinimumWidth(dte->sizeHint().width()
			+ dte->fontMetrics().width("999"));

	vl->addWidget(dte);
	dte->show();
	dte->setEnabled(false);

	QFont fnt(CO->valueFont());
//	fnt.setPointSize(AR3CMON_VALUE_FONT_SIZE);

	cb = new QCheckBox(trUtf8("Allow graduate adjustment (under 3min)"), this);
	cb->setFont(fnt);
	cb->setChecked(true);
	cb->setEnabled(false);
	vl->addWidget(cb);

	hl = new QHBoxLayout(vl);

	set = new QPushButton(trUtf8("Set"), this);
	set->setEnabled(false);
	connect(set, SIGNAL(clicked()), this, SLOT(set_clicked()));

	hl->addWidget(set);
	set->show();
	
	cancel = new QPushButton(tr("Cancel"), this);
	connect(cancel, SIGNAL(clicked()), this, SLOT(cancel_clicked()));
	hl->addWidget(cancel);
	cancel->show();
}

void SetTime::set_clicked()
{	
	QDateTime dt;
	dt = dte->dateTime();
	TheCommObject->setDataloggerTime(1000000000ULL * dt.toTime_t(), !cb->isChecked());
}

void SetTime::cancel_clicked()
{
	close();
}

void SetTime::check_pin(const QString& str)
{
	struct pin_action *a;

	if (str.length() > 10) {
		QMessageBox *mb = new QMessageBox("Warning", "How long you think the pin is?",
		                        QMessageBox::Warning, QMessageBox::Ok,
	        	                QMessageBox::NoButton, QMessageBox::NoButton,
	                	        this);
	        mb->show();
	}
	for (a = pin_action; a; a = a->next) {
		if (str != QString(a->pin))
			continue;
#ifndef __WIN32__
		if (!fork()) {
			execl(a->command, a->command, NULL);
			_exit(EXIT_FAILURE);
		}
#endif
		close();
	}
	if (str != QString(pin))
		return;

	le->setEnabled(false);
	key1->setEnabled(false);
	key2->setEnabled(false);
	key3->setEnabled(false);
	key4->setEnabled(false);
	key5->setEnabled(false);
	key6->setEnabled(false);
	key7->setEnabled(false);
	key8->setEnabled(false);
	key9->setEnabled(false);

	set->setEnabled(true);
	dte->setEnabled(true);
	cb->setEnabled(true);
}


void SetTime::key1_clicked()
{
	le->setText(le->text() + "1");
}

void SetTime::key2_clicked()
{
	le->setText(le->text() + "2");
}

void SetTime::key3_clicked()
{
	le->setText(le->text() + "3");
}

void SetTime::key4_clicked()
{
	le->setText(le->text() + "4");
}

void SetTime::key5_clicked()
{
	le->setText(le->text() + "5");
}

void SetTime::key6_clicked()
{
	le->setText(le->text() + "6");
}

void SetTime::key7_clicked()
{
	le->setText(le->text() + "7");
}

void SetTime::key8_clicked()
{
	le->setText(le->text() + "8");
}

void SetTime::key9_clicked()
{
	le->setText(le->text() + "9");
}



SetTime::~SetTime()
{
	delete cancel;
	delete set;
	delete dte;
	delete hl;
	delete vl;
	delete lb;
	delete le;
	delete h0;
	delete h1;
	delete h2;
	delete h3;
	delete key1;
	delete key2;
	delete key3;
	delete key4;
	delete key5;
	delete key6;
	delete key7;
	delete key8;
	delete key9;
}
