/*
 * ar3cmonitor - unix.c
 *
 * 	Support for UNIX
 * 
 * Copyright (C) 2010  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <stddef.h>

void show_screensaver(int);
void hide_screensaver(int);

static const struct sigaction sigusr1_action = {
	.sa_handler		= show_screensaver,
	.sa_flags		= SA_RESTART,
};

static const struct sigaction sigusr2_action = {
	.sa_handler		= hide_screensaver,
	.sa_flags		= SA_RESTART,
};

static const struct sigaction sigpipe_action = {
	.sa_handler		= SIG_IGN,
	.sa_flags		= SA_RESTART,
};

static void sigchdl_handler(int sig)
{
	int status;

	(void) sig;
	wait(&status);
}

static const struct sigaction sigchld_action = {
	.sa_handler		= sigchdl_handler,
	.sa_flags		= SA_RESTART,
};

void unix_init(void)
{
	sigaction(SIGUSR1, &sigusr1_action, NULL);
	sigaction(SIGUSR2, &sigusr2_action, NULL);
	sigaction(SIGPIPE, &sigpipe_action, NULL);
	sigaction(SIGCHLD, &sigchld_action, NULL);
}
